#include "System.h"
#include "Sprite.h"
#include "Dialog.h"
#include "BG.h"
#include "DataManager.h"

DataManager_Files	game_datas[] =
{
	{"town01_a.tim", 0},	// Tileset Image
	{"test.map", 0},		// Map
	{"dialog.tim", 0},		// Font Image
	{NULL, 0}
};

// These are corresponding index from the file above, it's easier to access using these name rather than index numbers
enum
{
	TOWNGFX,
	TESTMAP,
	DIALOGGFX
};

/******************************************/

BG	mybg[2];	// Our tilemap have 2 layers

GsIMAGE		Font;		// Font TIM
GsIMAGE		TileSet;	// Tileset TIM

P2D_DIALOG	diag;		// Our dialog box

/******************************************/

int			main(void)
{
	int		i;
	GsOT	*ot;
	int		cputime, gputime;

	// Init in 16bits 320x240
	System_Init(MODE_PAL, VMODE_16BITS, 320, 240, GsNONINTER, 0);

	// Load our files (from cd or from pc depending on CDROM_RELEASE flag compilation)
	DataManager_Init();
	DataManager_LoadDatas("DATA", game_datas);	// from the "DATA" folder

	// Load TIM Images in VRAM from our loaded data files
	Tim_Load(&Font, game_datas[DIALOGGFX].address);
	Tim_Load(&TileSet, game_datas[TOWNGFX].address);

	// Setup our tilemap from the TileSet image, and the map data
	BG_LoadMap(mybg, &TileSet, SPRITE_NORMAL, BG_CELL_16, game_datas[TESTMAP].address);

	// Setup our Dialog Box
	{
		P2D_BOX		mbox;
		GsSPRITE	a1, a2;

		// Init a black/blue gradient box
		Box_InitG(&mbox, BOX_VOPEN, PRIM2D_NORMAL, 16, 240-(64+8), 320-32, 64, 0x0000DD, 0x000000, 0x0000DD, 0x000000);
		// Set auto openning speed x/y of the box
		Box_SetOpenSpeed(&mbox, 6, 2);
		// Init the 2 sprites that will appear in the dialog box (cross and circle)
		Sprite_Init(&a1, &Font, SPRITE_NORMAL, 48, 24, 16, 15);	// These are coordinates and size of the sprite in the Font/Dialog image
		Sprite_Init(&a2, &Font, SPRITE_NORMAL, 32, 24, 16, 15);
		// Init our dialog box with options and our gradient box
		Dialog_Init(&diag, DIALOG_ANSWER_RIGHT | DIALOG_BVALID | DIALOG_BVALID2, 0, &mbox);
		// Set text for the dialog box
		Dialog_SetText(&diag, &Font, "Hello World !", 8,8, 8, false);
		// Set answer for the dialog box using the 2 previously setup sprite (and corresponding pad button to press)
		Dialog_SetAnswer(&diag, &a1, "OK", Pad1Cross, &a2, "No", Pad1Circle, 8);
	}

	// Main display loop (while not Start is pressed)
	while (!IsPadPress(Pad1Start))
	{
		// Get the OT to draw in
		ot = System_InitFrame();

		// Scroll the BG if we press arrows on the pad
		if (IsPadPress(Pad1Left))	BG_Scroll(&mybg[0], BG_BLOCK | BG_LEFT, 2), BG_Scroll(&mybg[1], BG_BLOCK | BG_LEFT, 2);
		if (IsPadPress(Pad1Right))	BG_Scroll(&mybg[0], BG_BLOCK | BG_RIGHT, 2), BG_Scroll(&mybg[1], BG_BLOCK | BG_RIGHT, 2);
		if (IsPadPress(Pad1Up))		BG_Scroll(&mybg[0], BG_BLOCK | BG_UP, 2), BG_Scroll(&mybg[1], BG_BLOCK | BG_UP, 2);
		if (IsPadPress(Pad1Down))	BG_Scroll(&mybg[0], BG_BLOCK | BG_DOWN, 2), BG_Scroll(&mybg[1], BG_BLOCK | BG_DOWN, 2);

		// Draw our 2 BG layer map
		BG_Draw(&mybg[0], ot);
		BG_Draw(&mybg[1], ot);

		// Draw dialog
		Dialog_Draw(&diag, ot);

		// Draw the Frame
		System_DrawFrame(ot, &cputime, &gputime);
	}

	System_Exit();
	return (0);
}
