#include "System.h"
#include "Sprite.h"
#include "DataManager.h"

// These are a group of files we want to load
DataManager_Files	game_datas[] =
{
	{"sprite.tim", 0},
	{"blur.tim", 0},
	{NULL, 0}
};

/******************************************/

GsIMAGE		TIMimage[2];	// Image TIM reference
GsSPRITE	TransparentSprite[4], MiddleSprite;	// Sprites

/******************************************/

int			main(void)
{
	int		i;
	GsOT	*ot;
	int		cputime, gputime;
	int		sens = 0, value = 0;

	// Init in 16bits 320x240
	System_Init(MODE_PAL, VMODE_16BITS, 320, 240, GsNONINTER, 0);

	// Load our files (from cd or from pc depending on CDROM_RELEASE flag compilation)
	DataManager_Init();
	DataManager_LoadDatas("DATA", game_datas);	// from the "DATA" folder

	// Load TIM Images in VRAM from our loaded data files
	Tim_Load(&TIMimage[0], game_datas[0].address);
	Tim_Load(&TIMimage[1], game_datas[1].address);

	// Init our middle sprite from the TIM image previously loaded in VRAM
	Sprite_Init(&MiddleSprite, &TIMimage[0], SPRITE_NORMAL, 0, 0, 96, 96);	// Position 0,0 and Size of the Sprite 96x96 inside the TIMimage
	Sprite_SetPosition(&MiddleSprite, (320-96)/2, (240-96)/2);	// Set the middle sprite position in the middle for the screen

	// Init our 4 transparent sprites from the TIM image previously loaded in VRAM
	for (i = 0; i < 4; i++)
		Sprite_Init(&TransparentSprite[i], &TIMimage[1], SPRITE_TRANS_100_PLUS_100, 0, 0, 128, 128);	// Position 0,0 and Size of the Sprite 128x128 inside the TIMimage
	Sprite_SetPosition(&TransparentSprite[0], 96-25, 56-25);	// Set our sprite in the 4 corner of the middle sprite
	Sprite_SetPosition(&TransparentSprite[1], 96+25, 56-25);
	Sprite_SetPosition(&TransparentSprite[2], 96-25, 56+25);
	Sprite_SetPosition(&TransparentSprite[3], 96+25, 56+25);

	// Main Display Loop until we press Start button on pad
	while (!IsPadPress(Pad1Start))
	{
		// This init our drawing, it return an OT pointer which is the table in which we will ask to draw
		ot = System_InitFrame();

		// Fade the transparent sprite back and forth
		for (i = 0; i < 4; i++)
			Sprite_Fade(&TransparentSprite[i], value);
		if (sens)
			value--;
		else
			value++;
		if (value == 128)
			sens = 1;
		else if (value == 0)
			sens = 0;

		// Ask to Draw the middle sprite and the 4 transparent sprites in the OT
		Sprite_DrawFast(&MiddleSprite, ot);
		for (i = 0; i < 4; i++)
			Sprite_DrawFast(&TransparentSprite[i], ot);

		// Ask to draw the OT on screen !
		System_DrawFrame(ot, &cputime, &gputime);	// You can print cputime and gputime to track how fast your program is, but you can also use NULL pointer if you don't mind.
	}

	System_Exit();
	return (0);
}
